// @ts-nocheck
import "./element-highlight.css";

import React, { useEffect, createRef } from "react";

const useCapture = true;
export const SELECTOR_ACTIVE = "contenteditable-element-selector-active";
export const ON_HOVER = "contenteditable-element-hover";
export const SELECTED = "contenteditable-element-seleted";

const shouldIgnoreElement = (ignoreElement, elementRef, target) => {
  if(elementRef && elementRef.current) {
    return elementRef.current.contains(target) === false;
  }

  if(ignoreElement) {
    return ignoreElement.contains(target) === false;
  }

  return false;
};

const cleanupSelected = () => {
  const selected = document.querySelector(`.${SELECTED}`);
  if (selected) selected.classList.remove(SELECTED);
}

const ElementHighlight = ({
  ignoreElement,
  isActiveRef,
  onSelect,
  children,
}) => {
  const elementRef = createRef();
  const activeRef = isActiveRef && isActiveRef.current;

  useEffect(() => {
    const mouseOutCallback = (event) => {
      const target: HTMLElement = event.target;
      if (!isActiveRef || !isActiveRef.current) return;
      target.classList.remove(ON_HOVER);
    };

    const mouseOverCallback = (event) => {
      const target: HTMLElement = event.target;
      if (!isActiveRef || !isActiveRef.current) return;
      if (shouldIgnoreElement(ignoreElement, elementRef, target)) return;
      target.classList.add(ON_HOVER);
    };

    const mouseClickCallback = (event) => {
      const target: HTMLElement = event.target;
      if (!isActiveRef || !isActiveRef.current) return;
      if (shouldIgnoreElement(ignoreElement, elementRef, target)) return;
      
      event.preventDefault();
      event.stopPropagation();

      cleanupSelected()
      target.classList.add(SELECTED);
      if(onSelect) onSelect(target);
    };

    window.addEventListener("mouseout", mouseOutCallback, useCapture);
    window.addEventListener("mouseover", mouseOverCallback, useCapture);
    window.addEventListener("click", mouseClickCallback, useCapture);

    return () => {
      window.removeEventListener("mouseout", mouseOutCallback, useCapture);
      window.removeEventListener("mouseover", mouseOverCallback, useCapture);
      window.removeEventListener("click", mouseClickCallback, useCapture);
    };
  }, [elementRef.current, activeRef, ignoreElement]);

  useEffect(() => {
    if (isActiveRef && isActiveRef.current) {
      const propKey = isActiveRef.current ? "add" : "remove";
      window.document.body.classList[propKey](SELECTOR_ACTIVE);
    }
  }, [activeRef]);

  return children ? (
    <elemetn-highlight ref={elementRef}>{children}</elemetn-highlight>
  ) : null;
};

export default ElementHighlight;
